deferred class ACTIVE_STATE_MACHINE
inherit
   STATE_MACHINE
   ACTIVABLE
      redefine
         init_activable
      end
   EXCEPTIONS
feature {ANY} -- From REACTIVE

   init_activable (name : STRING) is
      do
         !!message_queue.with_capacity(message_queue_capacity, 0)
         Precursor(name)
      end

   receive (e : EVENT) is
      do
         if message_queue.count >= message_queue_capacity then
            raise("QUEUE_FULL");
         else
            message_queue.put(e);
         end;
      end;

   dispatch is
      local
         current_event: EVENT;
      do
         current_event := message_queue.item;
         message_queue.remove;
         current_event.dispatch_to(Current);
      end;

feature {ANY} -- From ACTIVABLE

   nb_transitions : INTEGER is
         -- Number of outgoing transitions.
      do
         if not message_queue.empty then
            Result := 1;
         end;
      end -- nb_transitions

   labels (n : INTEGER) : STRING is
         -- Labels associated with transitions.
      require
         valid_transition: n > 0 and n <= nb_transitions
      do
         Result := ("?").twin;
         Result.append(message_queue.item.label);
      end -- labels

   guards (n : INTEGER) : BOOLEAN is
         -- Guards associated with transitions.
      require
         valid_transition: n > 0 and n <= nb_transitions
      do
         Result := True;
      end -- activable

   action (n : INTEGER) is
         -- Actions associated with fireable transitions.
      require
         fireable_transition: guards (n)
      local
         nb_completion : INTEGER
         is_discarded  : BOOLEAN
      do
         if not is_discarded then
            nb_completion := nb_transitions;
            if not message_queue.empty then
               nb_completion := nb_completion - 1;
            end;
            if n <= nb_completion then
               completion(n)
            else
               dispatch;
            end;
         end;
--      rescue
--         is_discarded := true;
--         retry;
      end -- action

   completion (n : INTEGER) is
         -- Actions associated with spontaneous transitions.
--      require
--         fireable_transition: guards (n)
      do
      end -- action

feature {CONFIGURATION}

   message_queue : LINKED_QUEUE[EVENT];

   message_queue_capacity : INTEGER is
      do
         Result := 1;
      ensure
         Result > 0;
      end -- message_queue_capacity

invariant
end -- class ACTIVE_STATE_MACHINE
