#define CAESAR_GRAPH_IMPLEMENTATION
#include "caesar_graph.h"
#include "signal.h"

void* eiffel_configuration_out(void* C);
T8 eiffel_string_to_external(void* C);
void eiffel_init_graph(void* C);
T6 eiffel_compare_label(void* C,void* a1,void* a2);
T6 eiffel_compare_state(void* C,void* a1,void* a2);
void eiffel_start_state(void* C,T8 a1);
void eiffel_iterate_state(void* C,T8 a1,void* a2,T8 a3,T8 a4);

/* define an array to store the last value of the ith signal */
static void * SIG_VALUE_TAB[NSIG];

void save_signal_handler() {
  int i;
  for (i=0; i<NSIG; i++) {              
    SIG_VALUE_TAB[i]=signal(i, SIG_DFL);
  }
}

void restore_signal_handler() {
  int i;
  for (i=0; i<NSIG; i++) {
    if (SIG_VALUE_TAB[i] != SIG_ERR) {
      /* check if the ith signal exists */ 
      signal(i, SIG_VALUE_TAB[i]);
    }
  }
}

static void * CURRENT_EIFFEL_SIMULATOR = 0;
void UMLAUT_SET_EIFFEL_SIMULATOR (SIMULATOR)
void * SIMULATOR;
{
      CURRENT_EIFFEL_SIMULATOR = SIMULATOR;
}


#ifndef CAESAR_OWN_GRAPH_COMPILER
CAESAR_TYPE_STRING CAESAR_GRAPH_COMPILER ()
{
     return ("UMLAUT");
}
#endif

#ifndef CAESAR_OWN_GRAPH_VERSION
CAESAR_TYPE_VERSION CAESAR_GRAPH_VERSION ()
{
     return (1.0);
}
#endif


#ifndef CAESAR_OWN_INIT_GRAPH
void CAESAR_INIT_GRAPH ()
{
  int argc = 0;
  char * argv[] = {"", 0};

  /* We must call Eiffel initialization code. */
  /* Before, we must save the signal's disposition initialized by CADP */
  /* and then, restore it to not execute the signal handlers initialized */
  /* by SmallEiffel*/
  save_signal_handler();
  initialize_eiffel_runtime(argc,argv);
  restore_signal_handler();
  eiffel_init_graph(eiffel_root_object);
}
#endif

#ifndef CAESAR_OWN_TYPE_STATE
typedef struct {CAESAR_TYPE_NATURAL HASH; void *MEMENTO;} CAESAR_BODY_STATE, *CAESAR_TYPE_STATE;
#endif

#ifndef CAESAR_OWN_SIZE_STATE
#undef CAESAR_SIZE_STATE
#undef CAESAR_HASH_SIZE_STATE
#define CAESAR_SIZE_STATE() (sizeof (CAESAR_BODY_STATE))
CAESAR_TYPE_NATURAL CAESAR_HINT_SIZE_STATE = CAESAR_SIZE_STATE ();
#define CAESAR_HASH_SIZE_STATE() (sizeof (CAESAR_TYPE_NATURAL))

CAESAR_TYPE_NATURAL CAESAR_HINT_HASH_SIZE_STATE = CAESAR_HASH_SIZE_STATE ();
#endif

#ifndef CAESAR_OWN_ALIGNMENT_STATE
CAESAR_TYPE_NATURAL CAESAR_HINT_ALIGNMENT_STATE = 4;
#endif

#ifndef CAESAR_OWN_COMPARE_STATE
CAESAR_TYPE_BOOLEAN CAESAR_COMPARE_STATE (CAESAR_S1, CAESAR_S2)
CAESAR_TYPE_STATE CAESAR_S1;
CAESAR_TYPE_STATE CAESAR_S2;
{
  int result;

  result = (eiffel_compare_state (CURRENT_EIFFEL_SIMULATOR, CAESAR_S1->MEMENTO, CAESAR_S2->MEMENTO));
  /*  if (result) {
    printf("State comparison of %p et %p: equal.\n", CAESAR_S1->MEMENTO, CAESAR_S2->MEMENTO);
  } else {
    printf("State comparison of %p et %p: different.\n", CAESAR_S1->MEMENTO, CAESAR_S2->MEMENTO);
    }*/
  return (result);
}
#endif

#ifndef CAESAR_OWN_HASH_STATE
CAESAR_TYPE_NATURAL CAESAR_HASH_STATE (CAESAR_S, CAESAR_MODULUS)
CAESAR_TYPE_STATE CAESAR_S;
CAESAR_TYPE_NATURAL CAESAR_MODULUS;
{
  /*  printf("HASH code of state %p : %d \n", CAESAR_S->MEMENTO, CAESAR_S->HASH); */
  return (CAESAR_S->HASH % CAESAR_MODULUS);
}
#endif

#ifndef CAESAR_OWN_MAX_FORMAT_STATE
CAESAR_TYPE_NATURAL CAESAR_MAX_FORMAT_STATE ()
{
     return (0);
}
#endif

#ifndef CAESAR_FORMAT_STATE
static CAESAR_TYPE_NATURAL CAESAR_CURRENT_STATE_FORMAT = 0;

void CAESAR_FORMAT_STATE (CAESAR_FORMAT)
CAESAR_TYPE_NATURAL CAESAR_FORMAT;
{
     if (CAESAR_FORMAT > CAESAR_MAX_FORMAT_STATE ()) {
          printf ("#203 bug during simulation:\n     unknown state printing format\n");
     } else
          CAESAR_CURRENT_STATE_FORMAT = CAESAR_FORMAT; 
}
#endif

#ifndef CAESAR_OWN_PRINT_STATE_HEADER
void CAESAR_PRINT_STATE_HEADER (CAESAR_FILE)
CAESAR_TYPE_FILE CAESAR_FILE;
{ 
     switch (CAESAR_CURRENT_STATE_FORMAT) {
     case 0:
          fprintf (CAESAR_FILE, "What the hell am I suppose to print here ?\n");
          break;
     default:
          printf ("#203 bug during simulation:\n     unknown state printing format\n");
          break;
     }
     
}
#endif

#ifndef CAESAR_OWN_PRINT_STATE
void CAESAR_PRINT_STATE (CAESAR_FILE, CAESAR_S)
CAESAR_TYPE_FILE CAESAR_FILE;
CAESAR_TYPE_STATE CAESAR_S;
{
  printf ("Calling Eiffel's state printing function.\n");
  fprintf (CAESAR_FILE, eiffel_string_to_external (eiffel_configuration_out (CAESAR_S->MEMENTO)));
}
#endif

#ifndef CAESAR_OWN_DELTA_STATE
void CAESAR_DELTA_STATE (CAESAR_FILE, CAESAR_S1, CAESAR_S2)
CAESAR_TYPE_FILE CAESAR_FILE;
CAESAR_TYPE_STATE CAESAR_S1;
CAESAR_TYPE_STATE CAESAR_S2;
{
     /* What should I print here ? */
}
#endif

#ifndef CAESAR_OWN_TYPE_LABEL
typedef struct {int HASH; void* LABEL;} CAESAR_BODY_LABEL, *CAESAR_TYPE_LABEL;
#endif

#ifndef CAESAR_OWN_SIZE_LABEL
#undef CAESAR_SIZE_LABEL
#define CAESAR_SIZE_LABEL() (sizeof (CAESAR_BODY_LABEL))
CAESAR_TYPE_NATURAL CAESAR_HINT_SIZE_LABEL = CAESAR_SIZE_LABEL ();
#endif

#ifndef CAESAR_OWN_HASH_SIZE_LABEL
#undef CAESAR_HASH_SIZE_LABEL
#define CAESAR_HASH_SIZE_LABEL() (1)
CAESAR_TYPE_NATURAL CAESAR_HINT_HASH_SIZE_LABEL = CAESAR_HASH_SIZE_LABEL ();
#endif

#ifndef CAESAR_OWN_ALIGNMENT_LABEL
CAESAR_TYPE_NATURAL CAESAR_HINT_ALIGNMENT_LABEL = 4;
#endif

#ifndef CAESAR_OWN_VISIBLE_LABEL
CAESAR_TYPE_BOOLEAN CAESAR_VISIBLE_LABEL (CAESAR_L)
CAESAR_TYPE_LABEL CAESAR_L;
{
  /* A bit rough... */
     return (1);
}
#endif

#ifndef CAESAR_OWN_GATE_LABEL
CAESAR_TYPE_STRING CAESAR_GATE_LABEL (CAESAR_L)
CAESAR_TYPE_LABEL CAESAR_L;
{
     return ("dummy-gate-label");
}
#endif

#ifndef CAESAR_OWN_RANK_LABEL
CAESAR_TYPE_NATURAL CAESAR_RANK_LABEL (CAESAR_L)
CAESAR_TYPE_LABEL CAESAR_L;
{
     return (0); /* ??? */
}
#endif

#ifndef CAESAR_COMPARE_LABEL
CAESAR_TYPE_BOOLEAN CAESAR_COMPARE_LABEL (CAESAR_L1, CAESAR_L2)
CAESAR_TYPE_LABEL CAESAR_L1;
CAESAR_TYPE_LABEL CAESAR_L2;
{
     return (eiffel_compare_label (CURRENT_EIFFEL_SIMULATOR, CAESAR_L1->LABEL, CAESAR_L2->LABEL));
}
#endif

#ifndef CAESAR_OWN_HASH_LABEL
CAESAR_TYPE_NATURAL CAESAR_HASH_LABEL (CAESAR_L, CAESAR_MODULUS)
CAESAR_TYPE_LABEL CAESAR_L;
CAESAR_TYPE_NATURAL CAESAR_MODULUS;
{
  /*     return (CAESAR_L->HASH); */
  return (1);
}
#endif

#ifndef CAESAR_OWN_PRINT_LABEL
void CAESAR_PRINT_LABEL (CAESAR_FILE, CAESAR_L)
CAESAR_TYPE_FILE CAESAR_FILE;
CAESAR_TYPE_LABEL CAESAR_L;
{
  fprintf (CAESAR_FILE, eiffel_string_to_external (CAESAR_L->LABEL));
}
#endif

#ifndef CAESAR_OWN_DUMP_LABEL
void CAESAR_DUMP_LABEL (CAESAR_STRING, CAESAR_L)
CAESAR_TYPE_STRING CAESAR_STRING;
CAESAR_TYPE_LABEL CAESAR_L;
{
  /* We should call eiffel's label.out */
  strcpy(CAESAR_STRING, eiffel_string_to_external (CAESAR_L->LABEL));
}
#endif

#ifndef CAESAR_OWN_STRING_LABEL
CAESAR_TYPE_STRING CAESAR_STRING_LABEL (CAESAR_L)
CAESAR_TYPE_LABEL CAESAR_L;
{
     return (eiffel_string_to_external (CAESAR_L->LABEL));
}
#endif

#ifndef CAESAR_OWN_MAX_FORMAT_LABEL
CAESAR_TYPE_NATURAL CAESAR_MAX_FORMAT_LABEL ()
{
     return (2);
}
#endif

#ifndef CAESAR_OWN_FORMAT_LABEL
static CAESAR_TYPE_NATURAL CAESAR_CURRENT_LABEL_FORMAT = 0;

void CAESAR_FORMAT_LABEL (CAESAR_FORMAT)
CAESAR_TYPE_NATURAL CAESAR_FORMAT;
{
     if (CAESAR_FORMAT > CAESAR_MAX_FORMAT_LABEL ()) {
          printf ("#204 bug during simulation:\n     unknown label printing format\n");
     } else
          CAESAR_CURRENT_LABEL_FORMAT = CAESAR_FORMAT;
}
#endif

#ifndef CAESAR_OWN_INFORMATION_LABEL
CAESAR_TYPE_STRING CAESAR_INFORMATION_LABEL (CAESAR_L)
CAESAR_TYPE_LABEL CAESAR_L;
{
     return ("");
}
#endif



#ifndef CAESAR_OWN_START_STATE
void CAESAR_START_STATE (CAESAR_S)
CAESAR_TYPE_STATE CAESAR_S;
{
     eiffel_start_state (CURRENT_EIFFEL_SIMULATOR, CAESAR_S);
}
#endif

#ifndef CAESAR_OWN_ITERATE_STATE
void CAESAR_ITERATE_STATE (CAESAR_S1, CAESAR_L, CAESAR_S2, CAESAR_LOOP)
CAESAR_TYPE_STATE CAESAR_S1;
CAESAR_TYPE_LABEL CAESAR_L;
CAESAR_TYPE_STATE CAESAR_S2;
#ifndef c_plusplus
void (*CAESAR_LOOP) ();
#else
void (*CAESAR_LOOP) (CAESAR_TYPE_STATE, CAESAR_TYPE_LABEL, CAESAR_TYPE_STATE);
#endif
{
  eiffel_iterate_state (CURRENT_EIFFEL_SIMULATOR, CAESAR_S1, CAESAR_L, CAESAR_S2, CAESAR_LOOP);
}
#endif

CAESAR_TYPE_NATURAL CAESAR_CARDINAL_LABEL (CAESAR_L)
CAESAR_TYPE_LABEL CAESAR_L;
{
  return 0;
}
