indexing
   description: "Eiffel part of the CADP connection";
   date: "$Date: 2000/11/01 18:31:42 $";
   revision: "$Revision: 1.13 $"
   author: "Alain Le Guennec"
deferred class CADP_SIMULATOR
inherit
   TIME_MANAGER_ACCESSOR
   EXCEPTIONS
   MEMORY
feature {ANY}

   make_cadp_simulator is
         -- Creation and Initialization
      do
         collection_on;
         full_collect;
         collection_off;
         !!exported_configurations.with_capacity(10000, 0);
         !!exported_labels.make
         nb_exported_configurations_since_last_gc_cycle := 0;
         UMLAUT_SET_EIFFEL_SIMULATOR (Current)
         --io.put_string ("Eiffel system initialized.%N")
         c_inline_h ("typedef (*loop)(void *, void *, void *);")
      end -- make_cadp_simulator

   initial_configuration : CONFIGURATION is
         -- Set the system in its initial configuration.
         -- UMLAUT shall generate a sub-class of CADP_SIMULATOR
         -- providing the realization of this routine,
         -- ideally based on a UML deployment diagram
         -- representing the initial configuration of the system.
      deferred
      end;

   current_configuration : CONFIGURATION;

feature {NONE}  -- The following features are to be called from CADP only.

   eiffel_start_state (START : POINTER) is
      local
         initial_state: CONFIGURATION;
         h            : INTEGER;
      do
         current_configuration := initial_configuration;
         export_configuration(current_configuration);
         h := current_configuration.hash;
         c_inline_c ("(int *)(((CAESAR_TYPE_STATE)a1)->MEMENTO) = (int *)(C->_current_configuration);")
         c_inline_c ("((CAESAR_TYPE_STATE)a1)->HASH = _h;")
	 --io.put_string ("CAESAR_START_STATE called...%N") 
      end

   eiffel_compare_state (S1, S2 : CONFIGURATION) : BOOLEAN is
      do
         Result := S1.is_deep_equal (S2)
--         io.put_string  ("Comparing two Eiffel states... ")
--         io.put_boolean (Result)
--         io.put_string  (".%N")
      end

   eiffel_compare_label (L1, L2 : STRING) : BOOLEAN is
      do
         Result := equal (L1, L2)
--         io.put_string  ("Comparing two Eiffel STRING labels... ")
--         io.put_boolean (Result)
--         io.put_string  (".%N")
      end

   eiffel_iterate_state (SP1: POINTER; LP : POINTER; SP3 : POINTER; F : POINTER) is
         -- This routine is to be exported as CAESAR_ITERATE_STATE.
         -- It receives 4 pointers:
         -- -the first  one points to the source state
         -- -the second one points to a memory area able to store any label
         -- -the third  one points to a memory area able to store any target state
         -- -the last   one points to a C-routine to be called for each target state,
         --  whose profile is:
         --  void (*) (CAESAR_TYPE_STATE, CAESAR_TYPE_LABEL, CAESAR_TYPE_STATE)
      local
         source         : CONFIGURATION;
         target         : CONFIGURATION;
         label          : STRING;

         was_discarded  : BOOLEAN
         p, action : INTEGER
         state_hash, label_hash : INTEGER;
      do
         if not was_discarded then
            c_inline_c ("_source = ((CAESAR_TYPE_STATE)a1)->MEMENTO;")
            p := source.process_table.lower
            action := 1;
         end;

         from
         until p > source.process_table.upper
         loop
            from
            until action > source.process_table.item (p).nb_transitions
            loop
               if source.process_table.item (p).guards (action) then
                  label := source.process_table.item (p).labels (action);
                  label.prepend(source.process_table.item (p).identity.out);
                  -- Label retrieving had better be side-effect free!

                  if not was_discarded then
                     current_configuration := deep_clone(source);
                  else
                     if not deep_equal(source, current_configuration) then
                        -- Roll-back if necessary (aborted transition)
                        current_configuration := deep_clone(source);
                     end;
                     was_discarded := false;
                  end;
                  current_configuration.process_table.item (p).action (action);

                  export_configuration(current_configuration);
                  -- We could used a label pool to prevent memory waste.

                  target := current_configuration;
                  state_hash := target.hash;
                  label := label_for_export(label);
                  label_hash := label.hash_code;
                  -- Calling the iterated C method.
                  c_inline_c ("(int *)(((CAESAR_TYPE_LABEL)a2)->LABEL)   = (int*)_label;")
                  c_inline_c ("((CAESAR_TYPE_LABEL)a2)->HASH = _label_hash;")
                  c_inline_c ("(int *)(((CAESAR_TYPE_STATE)a3)->MEMENTO) = (int*)_target;")
                  c_inline_c ("((CAESAR_TYPE_STATE)a3)->HASH = _state_hash;")
                  c_inline_c ("((loop)a4)(a1, a2, a3);")
               end -- if
               action := action + 1
            end -- loop
            p := p + 1
            action := 1;
         end -- loop
      rescue
         if is_developer_exception_of_name("QUEUE_FULL") then
            action := action + 1;
            was_discarded := true;
--            print("Transition discarded!%N");
            retry;
         elseif assertion_violation then
            action := action + 1;
            was_discarded := true;

            -- We select and export the special configuration.
            current_configuration := assertion_violation_configuration;
            label := label.twin
            label.append(" [assertion_violation]");

            export_configuration(current_configuration);
            -- We could used a label pool to prevent memory waste.

            target := current_configuration;
            state_hash := target.hash;
            label := label_for_export(label);
            label_hash := label.hash_code;
            -- Calling the iterated C method.
            c_inline_c ("(int *)(((CAESAR_TYPE_LABEL)a2)->LABEL)   = (int*)_label;")
            c_inline_c ("((CAESAR_TYPE_LABEL)a2)->HASH = _label_hash;")
            c_inline_c ("(int *)(((CAESAR_TYPE_STATE)a3)->MEMENTO) = (int*)_target;")
            c_inline_c ("((CAESAR_TYPE_STATE)a3)->HASH = _state_hash;")
            c_inline_c ("((loop)a4)(a1, a2, a3);")
            retry;
         else
            c_inline_c("print_exception();%N");
         end;
      end -- eiffel_iterate_state

   finalize_configuration(configuration : CONFIGURATION) is
         -- To be called by CADP applications
         -- to finalize a CAESAR_TYPE_STATE.
      require
         exported_configurations.has(configuration)
      do
         exported_configurations.remove(configuration);
         -- GC should be triggered after a sufficient number
         -- of configurations have been finalized. Not done.
      end;

   export_configuration(configuration : CONFIGURATION) is
         -- Called to prevent the GC from collecting exported objects.
      require
         configuration /= Void;
      do
         exported_configurations.add_last(configuration);
         -- We can't yet trigger the GC on finalization,
         -- but we can at least collect temporary objects (notably STRINGs)
         -- after a certain number of configurations have been exported.
         nb_exported_configurations_since_last_gc_cycle := nb_exported_configurations_since_last_gc_cycle + 1;
         if nb_exported_configurations_since_last_gc_cycle > 50000 then
            nb_exported_configurations_since_last_gc_cycle := 0;
            collection_on;
            full_collect;
            collection_off;
         end;

      end;

   label_for_export(label : STRING) : STRING is
         -- Called to prevent the GC from collecting exported objects.
         -- Labels are made unique, like in SE's STRING_ALIASER.
      require
         label /= Void;
      do
         if exported_labels.has(label) then
            Result := exported_labels.at(label);
         else
            Result := label.twin
            exported_labels.put(Result, Result);
         end;
      end;

feature {NONE}

   nb_exported_configurations_since_last_gc_cycle : INTEGER;

   exported_configurations : ARRAY[CONFIGURATION];
      -- Table used to store configurations exported
      -- through the CADP API, to prevent their collection.
      -- When finalization is eventually implemented in CADP,
      -- the finalize routine should simply remove
      -- finalized configurations out of this table.

   exported_labels : DICTIONARY[STRING,STRING];
      -- Table used to store labels exported
      -- through the CADP API, to prevent their collection.

   assertion_violation_configuration : CONFIGURATION is
      once
         !!Result.make
      end -- assertion_violation_configuration

   UMLAUT_SET_EIFFEL_SIMULATOR (s : CADP_SIMULATOR) is
      external "C_WithoutCurrent"
      alias "UMLAUT_SET_EIFFEL_SIMULATOR"
      end -- UMLAUT_SET_EIFFEL_SIMULATOR

end -- class CADP_SIMULATOR
