indexing
   description: ""
   note: ""
   status: "$State: Exp $"
   date: "$Date: 2000/11/02 14:32:45 $"
   revision: "$Revision: 1.1 $"
   author: "Alain Le Guennec"
deferred class ENUMERATION
inherit
   ANY
      redefine
	 out
      end;
feature {ANY} -- Queries

   out : STRING

   enumeration : STRING is
	 -- Returns the enumeration name of the current value.
	 -- The string should be cloned if modifications are intended.
         -- This routine should be made obsolete.
      require
	 not_invalid: is_initialized
      do
	 Result := out
      ensure
	 valid_enumeration: Result = out
      end -- enumeration

   value : like Current is
         -- For upward compatibility with old enumeration scheme.
         -- Should be mark obsolete ASAP.
      do
         Result := Current
      end -- value

   is_initialized : BOOLEAN is
         -- Should be mark obsolete as soon as it is possible
         -- to specify a default value for an enumeration type.
      do
         Result := not out.is_empty
      end -- is_initialized

   is_valid_enumeration(enum_name : STRING) : BOOLEAN is
	 -- Checks if the enum_name is a valid enumeration
      require
	 name_not_empty : enum_name /= Void -- and not enum_name.is_empty
      do
	 Result := all_images.has(enum_name)
      ensure
	 is_consistent: Result implies all_images.has(enum_name)
      end -- is_valid_enumeration

   value_of (enum_name : STRING) : like value is
      require
         enum_name_is_valid: is_valid_enumeration(enum_name)
      do
         !!Result.make;
         Result.from_string (enum_name);
      ensure
	 consistent: Result.out.is_equal(enum_name)
      end -- value_of

feature {ANY} -- Creation

   make is
      do
         out := "";
      ensure
         not is_initialized
      end -- make

   from_string, set_by_name (str : STRING) is
      require
         str /= Void
      local
         s : STRING
      do
         s := str.twin;
         s.right_adjust;
         s.left_adjust;
         s.to_lower;
         if all_images.has (s) then
            copy (all_images.at (s))
         else
            make
         end -- if
      check 
	 default_value: all_images.has (s) implies is_initialized
      end --check
      end -- from_string

   set (new_enum : like Current) is
         -- For upward compatibility with old enumeration scheme.
         -- Should be mark obsolete ASAP.
      do
         copy (new_enum)
      end

feature {ENUMERATION}

   set_image (new_image : STRING) is
      do
         out := new_image.twin;
      end -- set_image

feature {NONE} -- Private

   all_images : DICTIONARY[like Current, STRING] is
      deferred
      end -- all_images

end -- ENUMERATION
