indexing
   description: "Manage a bunch of timers for simulation purposes"
   note: "This is a Singleton: it should have a unique instance."
   status: "$State: Exp $"
   date: "$Date: 2000/01/06 16:57:11 $"
   revision: "$Revision: 1.1 $"
   author: "Jean-Marc Jezequel"
class TIME_MANAGER
inherit ACTIVABLE
creation {TIME_MANAGER_ACCESSOR} make 
   -- creation limited to TIME_MANAGER_ACCESSOR
feature {TIME_MANAGER_ACCESSOR} -- Creation
   make is
      -- Creation and initialization
      do
	 !!timer_list.make
	 date := 0
      end -- make

feature {ANY}
   reset (d : INTEGER) is
      do
         !!timer_list.make
         date := d
      end

feature {ANY} -- Queries
   date : INTEGER -- the current date in the system

   nb_transitions : INTEGER is
      do
         if not timer_list.empty then
            Result := 1
         else
            Result := 0
         end -- if
      end -- nb_transitions

   guards (n : INTEGER) : BOOLEAN is
         -- If there is a transition, it is fireable.
      do
         Result := True
      end -- guards

   labels (n : INTEGER) : STRING is
         -- If there is a transition, its label is that of the first timer.
      do
         Result := timer_list.first.label
      end -- labels
 
feature {ANY} -- Commands
   action (n : INTEGER) is
      -- fire the first timer in the list
      do
	 date := timer_list.first.deadline -- make time flies (like an arrow)
	 timer_list.first.fire
      end -- action
feature {TIMER}
   has (t : TIMER) : BOOLEAN is
	 -- Whether t is active
      require t_not_void: t /= Void
      do
	 Result := timer_list.has(t)
      end -- has
   remove (t : TIMER) is
	 -- remove t from the list of active timers
      require has_t: has(t)
      do
	 timer_list.start
	 timer_list.prune(t)
      ensure not_has_t: not has(t)
      end -- remove
   put (t : TIMER) is
	 -- add t to the list of active timers
      require not_has_t: not has(t)
      do
	 timer_list.extend(t)
      ensure has_t: has(t)
      end -- put
feature {NONE} -- Private
   timer_list : SORTED_TWO_WAY_LIST[TIMER]
invariant
   timer_list_not_void: timer_list /= Void
   positive_date: date >= 0
end -- TIME_MANAGER
