indexing 
   description: "Software version of an hardware timer"
   note: ""
   status: "$State: Exp $"
   date: "$Date: 2000/01/06 16:57:11 $"
   revision: "$Revision: 1.1 $"
   author: "Jean-Marc Jezequel"
class TIMER
inherit
   TIME_MANAGER_ACCESSOR -- Granting access to the TIME_MANAGER singleton
      undefine is_equal end;
   COMPARABLE
      redefine is_equal end
creation make, set, autotest
feature {ANY} -- Creation
   make (reactor : REACTIVE) is
	 -- Attach this unset timer to 'reactor' with a default alarm
      require
	 reactor_not_void: reactor /= Void
--	 not_is_running: not is_running
      do
	 target := reactor
	 !!alarm
      ensure 
	 not_is_running: not is_running
      end -- make
   set (reactor : REACTIVE; signal : ALARM; delay : INTEGER; new_label : STRING) is
	 -- set this timer to send 'signal' to 'reactor' after delay 'delay'
      require
	 not_is_running: not is_running
	 reactor_not_void: reactor /= Void
	 signal_not_void: signal /= Void
	 set_in_the_future: delay > 0
      do
	 target := reactor
	 alarm := signal
	 deadline := current_date + delay
         label := new_label
	 manager.put(Current)
      ensure 
	 is_running: is_running
	 deadline_set: deadline = current_date + delay
      end -- set
feature {ANY} -- Queries
   label : STRING
   deadline : INTEGER  -- expiration date 
   current_date : INTEGER is 
	 -- the date right now
      do
	 Result := manager.date
      end -- current_date
   is_running : BOOLEAN is
	 -- Whether this timer is active
      do
	 Result := manager.has(Current)
      end -- is_running
   is_ringing : BOOLEAN is
	 -- the deadline has been crossed
      do
	 Result := current_date >= deadline
      end -- is_ringing
   infix "<" (other: like Current) : BOOLEAN is
	 -- Is current object less than `other'?
     do
	if other = Void then 
	   Result := True -- Void is considered highest value
	else
	   Result := deadline < other.deadline
	end -- if
      end -- infix "<"
   is_equal (other: like Current) : BOOLEAN is
	 -- Is current object equal to `other' according to the deadline?
     do
	if other /= Void then 
	   Result := deadline = other.deadline
	end -- if
      end -- infix "<"
feature {ANY} -- Commands
   reset is
	 -- kill this timer
      require is_running: is_running
      do
	 manager.remove(Current)
      ensure inactive: not is_running
      end -- reset
   reschedule (new_delay : INTEGER; new_label : STRING) is
	 -- re-schedule the deadline of this timer to be 'new_delay'
      require in_the_future: new_delay > 0
      do
	 if is_running then reset end -- if
	 deadline := current_date + new_delay
         label := new_label
	 manager.put(Current)
      ensure 
	 is_running: is_running
	 deadline_set: deadline = current_date + new_delay
      end -- reschedule
feature {TIME_MANAGER} -- Only the time manager may access these
   fire is
	 -- reset itself and send an alarm to this timer target
      require
	 is_running: is_running
	 is_ringing: is_ringing
      do
	 reset
	 target.receive(alarm)
      end -- fire
feature {NONE} -- Protected
   target : REACTIVE  -- object that must be call upon firing
   alarm : ALARM -- event that must be sent to 'target' when firing
feature -- {TEST_DRIVER}
   autotest is
      -- self test this class
      do
	 manager.make -- Reset manager
	 test_make; manager.make 
	 test_set(3); manager.make 
	 test_infix_less (1,5) ; manager.make
	 test_reset; manager.make
	 test_reschedule; manager.make
	 test_fire
      end -- autotest
   test_make is
      local 
	 absorber : ABSORBER
      do
	 !!absorber
	 make(absorber)
      end -- test_make
   test_set (new_date : INTEGER) is
	 -- Set this timer to ring 
      local 
	 absorber : ABSORBER; a : ALARM
      do
	 !!absorber; !!a
	 set(absorber,a,new_date, "timeout") -- try to set this timer
	 check should_not_be_ringing: not is_ringing end
      end -- test_set
   test_infix_less (d1, d2 : INTEGER) is
      require d1 < d2
      local
	 absorber : ABSORBER; a : ALARM
	 other : TIMER
      do
	 test_set (d1) -- Set this timer to ring at d1
	 !!absorber; !!a
	 -- check is_less_than_void: Current < other end
	 !!other.set(absorber,a,d2,"timeout") -- set other to ring after Current
	 check is_less: Current < other end
      end -- test_infix_less
   test_reset is
      do
	 test_set (3) -- Set this timer to ring at 3
	 reset
      end -- test_reset
   test_reschedule is
      do
	 test_infix_less (3,4) 
	 -- Set this timer to ring at 3, and another one at 4
	 reschedule(2, "timeout") -- Reschedule Current 
	 reschedule(5, "timeout") -- Reschedule Current after other
      end -- test_reschedule
   test_fire is
      do
	 test_infix_less (3,4)
	 -- Set this timer to ring at 3, and another one at 4
	 manager.action (1) -- should fire this timer
	 check 
	    is_ringing: is_ringing
	    time_advanced: current_date = 3 
	 end -- check
	 reschedule(5, "timeout") -- set to ring again at 8
	 manager.action (1) -- should fire the other timer
         check 
	    time_advanced: current_date = 4 
	 end -- check
	 manager.action (1) -- should fire this timer again
	 check 
	    is_ringing: is_ringing
	    time_advanced: current_date = 8 
	 end -- check
      end -- test_fire

invariant
   target_not_void: target /= Void
   alarm_not_void: alarm /= Void
   positive_deadline: deadline >= 0
   is_running implies is_ringing = (current_date >= deadline)
end -- TIMER
